!---------------------------------------------------------------------------------------
! mod_ncdf
!---------------------------------------------------------------------------------------
!> mod_ncdf
!!
!---------------------------------------------------------------------------------------


module mod_ncdf

  use mod_var
  use mod_settings
  use netcdf

  implicit none
  private

  public :: read_ncdf, write_ncdf, check

  contains

  ! --------------------------------------------------
  ! read_ncdf
  ! --------------------------------------------------

  subroutine read_ncdf(files, config, filename, varname, datain)

    implicit none

    type(files_t),                        intent(in)     :: files
    type(config_t),                       intent(in)     :: config
    character(len=max_path_len),          intent(in)     :: filename
    character(len=max_name_len),          intent(in)     :: varname
    real, dimension(nxregrid,nyregrid),   intent(in out) :: datain

    character(len=max_name_len)                          :: lonname, latname, timename
    logical                                              :: lexist
    integer                                              :: ncid, lon_dimid, lat_dimid, time_dimid, ncerr
    integer                                              :: varid, lonid, latid
    integer                                              :: nlon, nlat, nt
    integer                                              :: ix, jy
    real, dimension(:), allocatable                      :: lon, lat
    real, dimension(:,:,:), allocatable                  :: flx

    lonname = files%lonname_flx
    latname = files%latname_flx
    timename = files%timename_flx

    inquire(file=trim(filename),exist=lexist)
    if (.not.lexist) then
      write(*,*) 'ERROR: cannot find '//trim(filename)
      stop
    endif

    write(*,*) 'Reading file: '//trim(filename)

    call check( nf90_open(trim(filename),nf90_NOWRITE,ncid) )
    call check( nf90_inq_dimid(ncid,trim(lonname),lon_dimid) )
    call check( nf90_inq_dimid(ncid,trim(latname),lat_dimid) )
    ncerr = nf90_inq_dimid(ncid,trim(timename),time_dimid) 
    if ( ncerr.lt.0 ) write(*,*) 'WARNING: file has no time dimension'

    call check( nf90_inquire_dimension(ncid,lon_dimid,len=nlon) )
    call check( nf90_inquire_dimension(ncid,lat_dimid,len=nlat) )
    if ( ncerr.eq.0 ) then
      call check( nf90_inquire_dimension(ncid,time_dimid,len=nt) )
    else
      nt = 1
    endif

    print*, 'read_ncdf: nlon, nlat, nt =', nlon, nlat, nt

    allocate ( lon(nlon) )
    allocate ( lat(nlat) )
    allocate ( flx(nxregrid,nyregrid,nt) )

    call check( nf90_inq_varid(ncid,trim(lonname),lonid) )
    call check( nf90_get_var(ncid,lonid,lon) )
    call check( nf90_inq_varid(ncid,trim(latname),latid) )
    call check( nf90_get_var(ncid,latid,lat) )

    ! indices to variable
    ix = minloc(lon, dim=1, mask=abs(lon-dx/2.-rllx).eq.minval(abs(lon-dx/2.-rllx)))
    jy = minloc(lat, dim=1, mask=abs(lat-dy/2.-rlly).eq.minval(abs(lat-dy/2.-rlly)))

    ! check flux extent
    if ( (ix.eq.0).or.(jy.eq.0) ) then
      write(*,*) 'ERROR: incompatible domain in '//trim(filename)
      stop
    endif
    if ( (nlon.lt.(ix+nxregrid-1)).or.(nlat.lt.(jy+nyregrid-1)) ) then
      write(*,*) 'ERROR: incompatible dimensions in '//trim(filename)
      stop
    endif

    ! read flux
    call check( nf90_inq_varid(ncid,trim(varname),varid) )
    if ( ncerr.eq.0 ) then
      call check( nf90_get_var(ncid,varid,flx,start=(/ix,jy,1/),count=(/nxregrid,nyregrid,nt/)) )
    else
      call check( nf90_get_var(ncid,varid,flx(:,:,1),start=(/ix,jy/),count=(/nxregrid,nyregrid/)) )
    endif
    call check( nf90_close(ncid) )

    deallocate(lon)
    deallocate(lat)

    if ( ncerr.eq.0 ) then
      datain = sum(flx,dim=3)/real(nt)
    else
      datain = flx(:,:,1)
    endif

    deallocate(flx)

  end subroutine read_ncdf

  ! --------------------------------------------------
  ! write_ncdf
  ! --------------------------------------------------

  subroutine write_ncdf(files, filename, varname, datain)

    implicit none

    type(files_t),                      intent(in) :: files
    character(len=max_path_len),        intent(in) :: filename
    character(len=max_name_len),        intent(in) :: varname
    real, dimension(nxregrid,nyregrid), intent(in) :: datain

    character(len=max_name_len)                    :: lonname, latname
    character(len=*), parameter                    :: varunit = 'none'
    character(len=*), parameter                    :: dimunit = 'degrees'
    character(len=*), parameter                    :: units = 'unit'
    integer                                        :: ncid 
    integer                                        :: lon_dimid, lat_dimid
    integer                                        :: varid, lon_varid, lat_varid 
    integer, dimension(2)                          :: dimids

    lonname = files%lonname_regs
    latname = files%latname_regs

    call check( nf90_create(trim(filename), nf90_clobber, ncid) )

    call check( nf90_def_dim(ncid, lonname, nxregrid, lon_dimid) )
    call check( nf90_def_var(ncid, lonname, nf90_real, lon_dimid, lon_varid) )
    call check( nf90_put_att(ncid, lon_varid, units, dimunit) )
    call check( nf90_def_dim(ncid, latname, nyregrid, lat_dimid) )
    call check( nf90_def_var(ncid, latname, nf90_real, lat_dimid, lat_varid) )
    call check( nf90_put_att(ncid, lat_varid, units, dimunit) )

    dimids = (/ lon_dimid, lat_dimid /)
    call check( nf90_def_var(ncid, varname, nf90_real, dimids, varid) )
    call check( nf90_put_att(ncid, varid, units, varunit) )

    call check( nf90_enddef(ncid) )

    call check( nf90_put_var(ncid, lon_varid, reg_lon) )
    call check( nf90_put_var(ncid, lat_varid, reg_lat) )
    call check( nf90_put_var(ncid, varid, datain) )

    call check( nf90_close(ncid) )

  end subroutine write_ncdf

  ! --------------------------------------------------
  ! check
  ! --------------------------------------------------

  subroutine check(status)

    integer, intent(in) :: status

    if ( status.ne.nf90_noerr ) then
      write(*,*) trim(nf90_strerror(status))
      stop
    end if

  end subroutine check

  ! --------------------------------------------------


end module mod_ncdf


