!---------------------------------------------------------------------------------------
! FLEXINVERT: average_fp
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> average_fp
!!
!! Purpose:    Averages footprints over multiple releases. For instance;
!!             averaging 24 hourly releases to get the footprint that is representative 
!!             for observations averaged over 1 day 
!!             Note: this also requires that the file with observations includes an 
!!             averaging or total time per observation. Added to allow more flexibility 
!!             using the same footprints for different types of simulations and projects.
!!
!! Interface:
!!
!!    Inputs
!!             files   -  file data structure
!!             config  -  configuration settings data structure
!!             obs     -  observation data structure
!!
!!    Externals
!!             caldate
!!             get_nread
!!             read_dates
!!             read_header
!!             read_grid
!!             save_average_fp
!!
!---------------------------------------------------------------------------------------

subroutine average_fp(files, config, obs)

  use mod_flexpart
  use mod_settings
  use mod_var
  use mod_dates
  use mod_obs
  use mod_tools
  use mod_save, only : save_average_fp

  implicit none

  type (files_t),          intent (in)     :: files
  type (config_t),         intent (in)     :: config
  type (obs_t),            intent (in out) :: obs

  character(len=max_path_len)              :: path_flexrec, subdir_fp, filename, filenest, filename_out
  character(len=max_path_len)              :: filedates 
  character(len=6)                         :: adate
  character(len=8)                         :: areldate, areltime
  logical                                  :: lexist
  integer                                  :: i
  integer                                  :: jjjjmmdd, hhmiss
  integer                                  :: month, prevmonth
  integer                                  :: countrate
  integer                                  :: tread1, tread2

  real, dimension(nxgrid,nygrid,maxngrid)  :: grid                        ! flexpart flux sensitivity (or footprints)
  real, dimension(nxgrid,nygrid,maxngrid)  :: grid_tmp                    ! flexpart flux sensitivity (or footprints), temporary to later calculate average
  real, dimension(nnxgrid,nnygrid,maxngrid):: gridnest                    ! flexpart nest flux sensitivity (or footprints)
  real, dimension(nnxgrid,nnygrid,maxngrid):: gridnest_tmp                ! flexpart nest flux sensitivity (or footprints), temporary to later calculate average
  real(kind=8), dimension(maxngrid)        :: gtime                       ! flux sensitivity time stamp (julian days)
  real(kind=8), dimension(maxngrid)        :: gtime_tmp                   ! flux sensitivity time stamp (julian days) of first footprint per observation
  integer                                  :: ngrid

  real                                     :: bndx, bndy, delx, dely
  integer                                  :: numx, numy, xshift, new_grid_time, standard_grid_time
  integer                                  :: numpoint, release_nr, nr_footprints
  integer                                  :: ibdate, ibtime
  real(kind=8), dimension(maxpoint,2)      :: releases

  ! loop over observations
  ! ----------------------

  prevmonth = 0

  do i = 1, nobs

    ! reset values
    gtime_tmp(:) = 0d0
    grid_tmp(:,:,:) = 0.
    gridnest_tmp(:,:,:) = 0.
    nr_footprints = 0

    ! Check month of observation
    call caldate(obs%jdate(i), jjjjmmdd, hhmiss)
    write(logid,*) 'Get footprints for observation nr:', i, obs%jdate(i), jjjjmmdd, hhmiss
    month = jjjjmmdd/100
    write(adate,'(I6.6)') jjjjmmdd/100
    path_flexrec = trim(files%path_flexpart)//trim(obs%recs(i))//'/'//trim(adate)//'/'
    filename = trim(path_flexrec)//'header'

    ! check if netcdf file for this observation already exists
    write(areldate,'(I8.8)') jjjjmmdd
    write(areltime,'(I6.6)') hhmiss
    inquire ( file=trim(files%path_flexncdf)//trim(obs%recs(i))//'/'//trim(adate)//'/'//&
                   'grid_time_'//trim(areldate)//trim(areltime)//'_001.nc', exist=lexist )
    if ( lexist ) go to 10

    ! Get header and release information related to current observation location 
    ! (if not read and same for previous observation)
    if ( i.eq.1 .or. month.ne.prevmonth ) then
      write(logid,*) 'Get header:', i, ' '//trim(path_flexrec)//'header'
      call read_header(filename, numpoint, ibdate, ibtime, releases, &
                         numx, numy, bndx, bndy, delx, dely, xshift)
    else
      if(trim(obs%recs(i)).ne.trim(obs%recs(i-1)))then
        write(logid,*) 'Get header:', i, ' '//trim(path_flexrec)//'header'
        call read_header(filename, numpoint, ibdate, ibtime, releases, &
                           numx, numy, bndx, bndy, delx, dely, xshift)
      endif
    endif
  
    ! Find the first release number corresponding to the current observation
    release_nr=1
    do while ( int(releases(release_nr,1)*1e4,kind=8).lt.int(obs%jdate(i)*1e4,kind=8) .and. release_nr.lt.numpoint )
      release_nr = release_nr+1
    end do

    ! define first filename
    call caldate(releases(release_nr,1), jjjjmmdd, hhmiss)
    ! round seconds
    hhmiss = (hhmiss/100)*100

    write(areldate,'(I8.8)') jjjjmmdd
    write(areltime,'(I6.6)') hhmiss

    filename = trim(path_flexrec)//'grid_time_'//trim(areldate)//trim(areltime)//'_001'
    filenest = trim(path_flexrec)//'grid_time_nest_'//trim(areldate)//trim(areltime)//'_001'
    filedates = trim(path_flexrec)//'dates'
 
    ! read flexpart footprints
    !-------------------------

    inquire ( file=trim(filename), exist=lexist )
    if ( lexist ) then
      write(logid,*) 'Reading flexpart :'//trim(filename)
      call system_clock(tread1, countrate)

      ! Read first grid_time file from start of release
      call read_grid(filename, filedates, obs%jdate(i), nxgrid, nygrid, nxshift, ngrid, grid, gtime)
      gtime_tmp = gtime  ! Save the output date of each field in footprint grid
      grid_tmp = grid    ! Save the footprint to temporary grid for summation
      nr_footprints = 1  ! Start counting number of footprints to be averaged   
      print*, 'average_fp: gtime = ', gtime

      ! Same for nested grid if applicable
      if ( config%nested ) then
        ! read nested footprints
        inquire ( file=trim(filenest),exist=lexist )
        if ( lexist ) then
          write(logid,*) 'Reading flexpart nest :'//trim(filenest)
          call read_grid(filenest, filedates, obs%jdate(i), nnxgrid, nnygrid, nnxshift, ngrid, gridnest, gtime)
        else
          write(logid,*) 'WARNING: cannot find '//trim(filename)
          go to 10
        endif
        gridnest_tmp = gridnest ! Save the footprint to temporary grid for summation
      endif

      ! Current release_nr was already read, start with next
      release_nr = release_nr+1 
      write(logid,*) 'average_fp: obs%jdate(i)+obs%avetime(i) = ',obs%jdate(i)+obs%avetime(i)
      write(logid,*) 'average_fp: releases(release_nr,2) = ', releases(release_nr,2)
      do while ( int(releases(release_nr,2)*1e4,kind=8).le.int((obs%jdate(i)+obs%avetime(i))*1e4,kind=8) .and. release_nr.lt.numpoint )

        call caldate(releases(release_nr,1), jjjjmmdd, hhmiss)
        month = jjjjmmdd/100
        write(adate,'(I6.6)') jjjjmmdd/100
        write(areldate,'(I8.8)') jjjjmmdd
        hhmiss=int(hhmiss/100)*100
        write(areltime,'(I6.6)') hhmiss
        path_flexrec = trim(files%path_flexpart)//trim(obs%recs(i))//'/'//trim(adate)//'/'
        filename = trim(path_flexrec)//'grid_time_'//trim(areldate)//trim(areltime)//'_001'
        filenest = trim(path_flexrec)//'grid_time_nest_'//trim(areldate)//trim(areltime)//'_001'
        filedates = trim(path_flexrec)//'dates'

        write(logid,*) 'Reading flexpart:'//trim(filename)
        call read_grid(filename, filedates, obs%jdate(i), nxgrid, nygrid, nxshift, ngrid, grid, gtime)
        nr_footprints=nr_footprints+1

        ! Same for nested grid if applicable
        if ( config%nested ) then
          write(logid,*) 'Reading flexpart nest:'//trim(filenest)
          call read_grid(filenest, filedates, obs%jdate(i), nnxgrid, nnygrid, nnxshift, ngrid, gridnest, gtime)
        endif

        release_nr = release_nr+1

        ! Sum the grid with previous footprints
        ! Check for corresponding output date in gtime
        do standard_grid_time = 1, maxngrid
          do new_grid_time = 1, maxngrid
            if ( gtime(new_grid_time) .eq. gtime_tmp(standard_grid_time)) then
              grid_tmp(:,:,standard_grid_time) = grid_tmp(:,:,standard_grid_time)+grid(:,:,new_grid_time)
              if ( config%nested ) then
                gridnest_tmp(:,:,standard_grid_time)=gridnest_tmp(:,:,standard_grid_time)+gridnest(:,:,new_grid_time)
              endif
            endif
          end do
        end do

      end do ! Finished summing footprints to get average

      ! Replace grid by average grid and save
      ! -------------------------------------

      write(logid,*) 'Total nr footprints for this observation:', nr_footprints
      grid = grid_tmp/real(nr_footprints)
      gridnest = gridnest_tmp/real(nr_footprints)
      gtime = gtime_tmp
      print*, 'average_fp: gtime after average = ',gtime

      ! Save as netcdf
      ! output filename same as start date and time of observation
      call caldate(obs%jdate(i), jjjjmmdd, hhmiss)
      write(adate,'(I6.6)') jjjjmmdd/100
      write(areldate,'(I8.8)') jjjjmmdd
      write(areltime,'(I6.6)') hhmiss
      ! create subdirectory in path_flexncdf for files
      subdir_fp = trim(obs%recs(i))//'/'//trim(adate)//'/'
      call system('mkdir -p '//trim(files%path_flexncdf)//trim(obs%recs(i)))
      call system('mkdir -p '//trim(files%path_flexncdf)//trim(subdir_fp))
      filename_out = trim(subdir_fp)//'grid_time_'//trim(areldate)//trim(areltime)//'_001.nc'
      call save_average_fp(files, filename_out, nxgrid, nygrid, maxngrid, ngrid, grid, gbl_lon, gbl_lat, gtime)
      ! If applicable same for nested grid
      if ( config%nested ) then 
        filename_out = trim(subdir_fp)//'grid_time_nest_'//trim(areldate)//trim(areltime)//'_001.nc'
        call save_average_fp(files, filename_out, nnxgrid, nnygrid, maxngrid, ngrid, gridnest, nest_lon, nest_lat, gtime)
      endif

      call system_clock(tread2, countrate)

      write(logid,*) 'average_fp: time to average grid (s) = ',(tread2-tread1)/countrate

      prevmonth = month 

    ! If filename not found 
    else
      write(logid,*) 'WARNING: cannot find '//trim(filename)
      go to 10
    endif

10 continue

  end do ! loop over observations


end subroutine average_fp


