!---------------------------------------------------------------------------------------
! FLEXINVERT: init_cini_month
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> init_cini_month
!!
!! Purpose:    Calculates the initial concentration for each observation using the
!!             4D initial concentration fields and the flexpart grid_initial files.
!!
!! Interface:
!!
!!    Inputs
!!             files   -  file data structure
!!             config  -  configuration settings data structure
!!             obs     -  observation data structure 
!!
!!    Externals
!!             caldate
!!             read_init              
!!             get_initconc
!!
!---------------------------------------------------------------------------------------

subroutine init_cini_month(files, config, obs)

  use mod_flexpart
  use mod_settings
  use mod_strings
  use mod_var
  use mod_dates
  use mod_obs
  use mod_tools

  implicit none

  type (files_t),        intent (in)     :: files
  type (config_t),       intent (in)     :: config
  type (obs_t),          intent (in out) :: obs

  character(max_path_len)                :: path_flexrec, filename, pathname
  character(max_name_len)                :: varname
  character(len=8)                       :: adate, areldate, areltime
  character(len=6)                       :: ayear
  logical                                :: lexist
  integer                                :: lastjjjjmm, prejjjjmm, projjjjmm
  integer                                :: jjjjmm, jjjjmmdd, hhmiss, mm, eomday
  integer                                :: ix, jy, kz, i, n, nz, ntime
  integer                                :: month, prevmonth
  real                                   :: conc
  real(kind=8)                           :: jdmid, jdpre, jdlast
  real(kind=8), dimension(nobs)          :: jdates
  real, dimension(nobs,ncini)            :: cini
  integer, dimension(nobs)               :: ind
  real, dimension(nxgrid,nygrid,nzgrid)  :: gridini, gridini_tmp
  real, dimension(nxgrid,nygrid,nzgrid,1):: concini, preconcini, proconcini

  real                                   :: bndx, bndy, delx, dely
  integer                                :: numx, numy, xshift
  integer                                :: numpoint, release_nr, nr_init
  integer                                :: ibdate, ibtime
  real(kind=8), dimension(maxpoint,2)    :: releases
  integer                                :: nretr, ierr
  character(len=6)                       :: anretr

  ! loop over observations
  ! ----------------------

  lastjjjjmm = 0
  concini(:,:,:,:) = 0.
  preconcini(:,:,:,:) = 0.
  proconcini(:,:,:,:) = 0.
  cini(:,:) = 0.
  prevmonth = 0
  ntime = 1

  ! sort obs chronologically for efficiency
  jdates = obs%jdate(:)
  call sort(nobs,jdates,ind)

  do i = 1, nobs

    ! reset values
    gridini(:,:,:) = 0.
    nr_init = 0

    ! check month of observation
    call caldate(jdates(i), jjjjmmdd, hhmiss)
    write(adate,'(I6.6)') jjjjmmdd/100
    write(areldate,'(I8.8)') jjjjmmdd
    month = jjjjmmdd/100
    path_flexrec = trim(files%path_flexpart)//trim(obs%recs(ind(i)))//'/'//trim(adate)//'/'

    if ( config%average_fp ) then

      ! match and, if needed, average grid_initial files to observation timestamp

      ! get header and release information related to current observation location 
      ! (if not already read and same for previous observation)
      filename = trim(path_flexrec)//'header'
      if ( i.eq.1 .or. month.ne.prevmonth ) then
        write(logid,*) 'Get header:', i, ' '//trim(path_flexrec)//'header'
        call read_header(filename, numpoint, ibdate, ibtime, releases, &
                           numx, numy, bndx, bndy, delx, dely, xshift)
      else
        if(trim(obs%recs(ind(i))).ne.trim(obs%recs(ind(i-1))))then
          write(logid,*) 'Get header:', i, ' '//trim(path_flexrec)//'header'
          call read_header(filename, numpoint, ibdate, ibtime, releases, &
                             numx, numy, bndx, bndy, delx, dely, xshift)
        endif
      endif
      ! find the first release number corresponding to the current observation
      release_nr=1
      do while ( int(releases(release_nr,1)*1.e4,kind=8).lt.int(jdates(i)*1.e4,kind=8) .and. release_nr.lt.numpoint )
        release_nr = release_nr+1
      end do
      ! define first filename
      call caldate(releases(release_nr,1), jjjjmmdd, hhmiss)
      ! round seconds
      hhmiss = (hhmiss/100)*100
      write(areldate,'(I8.8)') jjjjmmdd
      write(areltime,'(I6.6)') hhmiss
      filename = trim(path_flexrec)//'grid_initial_'//trim(areldate)//trim(areltime)//'_001'
      ! read first flexpart grid_init files
      inquire ( file=trim(filename), exist=lexist )
      if ( lexist ) then
        call read_init(filename, gridini_tmp)
        gridini = gridini + gridini_tmp
        nr_init = 1
        ! current release_nr was already read, start with next
        release_nr = release_nr + 1
        do while ( int(releases(release_nr,2)*1.e4,kind=8).le.int((jdates(i)+obs%avetime(ind(i)))*1.e4,kind=8) .and. release_nr.lt.numpoint )
          call caldate(releases(release_nr,1), jjjjmmdd, hhmiss)
          write(adate,'(I6.6)') jjjjmmdd/100
          write(areldate,'(I8.8)') jjjjmmdd
          hhmiss=int(hhmiss/100)*100
          write(areltime,'(I6.6)') hhmiss
          path_flexrec = trim(files%path_flexpart)//trim(obs%recs(ind(i)))//'/'//trim(adate)//'/'
          filename = trim(path_flexrec)//'grid_initial_'//trim(areldate)//trim(areltime)//'_001'
          ! sum the grid with previous initial grids
          call read_init(filename, gridini_tmp)
          gridini = gridini + gridini_tmp
          release_nr = release_nr+1
          nr_init = nr_init + 1
        end do ! finished summing grid_initial
        ! convert from ppt to fraction of one
        gridini = gridini*1.e-12
        gridini = gridini/real(nr_init)
      else
        ! filename not found 
        write(logid,*) 'WARNING: cannot find '//trim(filename)
        go to 10
      endif

    else

      ! timestamps of grid_init files match observations

      ! define file name
      write(areldate,'(I8.8)') jjjjmmdd
      write(areltime,'(I6.6)') hhmiss
      filename = trim(path_flexrec)//'grid_initial_'//trim(areldate)//trim(areltime)//'_001'
      print*, 'init_cini_month: filename = ',filename
      ! read flexpart grid_init file
      inquire(file=trim(filename),exist=lexist)
      if(lexist) then
        call read_init(filename, gridini)
        ! convert from ppt to fraction of one
        gridini = gridini*1.e-12
      else
        write(logid,*) 'WARNING: cannot find ',filename
        go to 10
      endif

    endif 

    ! calculate termination time of trajectory
    call caldate(jdates(i) - trajdays, jjjjmmdd, hhmiss)
    jjjjmm = jjjjmmdd/100
   
    ! preceding and proceding months
    mm = jjjjmm - (jjjjmm/100)*100
    if( mm.gt.1 ) then
      prejjjjmm = jjjjmm - 1
    else
      prejjjjmm = (jjjjmm/100 - 1)*100 + 12
    endif
    if( mm.lt.12 ) then
      projjjjmm = jjjjmm + 1
    else
      projjjjmm = (jjjjmm/100 + 1)*100 + 1
    endif

    jdmid = juldate(jjjjmm*100+14, 0)
    jdpre = juldate(prejjjjmm*100+14, 0)
    jdlast = juldate(lastjjjjmm*100+14, 0) 
    eomday = calceomday(prejjjjmm)

    ! get new concentrations if not same month as previous observation 
    if( jjjjmm.ne.lastjjjjmm ) then
      if( (lastjjjjmm.ne.0).and.(jdmid.eq.(jdlast+dble(eomday))) ) then
        ! this is proceeding month
        preconcini = concini
        concini = proconcini
        ! get proceeding concentration field
        write(adate,'(I6.6)') projjjjmm
        write(ayear,'(I4.4)') projjjjmm/100
        select case ( files%init_type )
          case (4)
            pathname = str_replace(files%path_initconc, 'YYYY', ayear)
            filename = str_replace(files%file_initconc, 'YYYYMM', adate)
            filename = trim(pathname)//trim(filename)
          case (5)
            filename = str_replace(files%file_initconc, 'YYYYMM', adate)  
            filename = trim(files%path_initconc)//trim(filename)
        end select
        inquire(file=trim(filename),exist=lexist)
        if(lexist) then
          varname = files%varname_init
          call get_initconc(files, config, filename, ntime, proconcini)
        else
          write(logid,*) 'WARNING: cannot find ',filename
          write(logid,*) 'WARNING: next background file missing, copy present into next to avoid 0 value'
          proconcini = concini
        endif
      else 
        ! get current concentration field
        write(adate,'(I6.6)') jjjjmm
        write(ayear,'(I4.4)') jjjjmm/100
        select case ( files%init_type )
          case (4)
            pathname = str_replace(files%path_initconc, 'YYYY', ayear)
            filename = str_replace(files%file_initconc, 'YYYYMM', adate)
            filename = trim(pathname)//trim(filename)
          case (5)
            filename = str_replace(files%file_initconc, 'YYYYMM', adate)
            filename = trim(files%path_initconc)//trim(filename)
        end select
        inquire(file=trim(filename),exist=lexist)
        if(lexist) then
          varname = files%varname_init
          call get_initconc(files, config, filename, ntime, concini)
        else
          write(logid,*) 'WARNING: cannot find ',filename
          go to 10
        endif
        ! get previous concentration field
        write(adate,'(I6.6)') prejjjjmm
        write(ayear,'(I4.4)') prejjjjmm/100
        select case ( files%init_type )
          case (4)
            pathname = str_replace(files%path_initconc, 'YYYY', ayear)
            filename = str_replace(files%file_initconc, 'YYYYMM', adate)
            filename = trim(pathname)//trim(filename)
          case (5)
            filename = str_replace(files%file_initconc, 'YYYYMM', adate)
            filename = trim(files%path_initconc)//trim(filename)
        end select
        inquire(file=trim(filename),exist=lexist)
        if(lexist) then
          varname = files%varname_init
          call get_initconc(files, config, filename, ntime, preconcini)
        else
          write(logid,*) 'WARNING: cannot find ',filename
          go to 10
        endif
        ! get next concentration field
        write(adate,'(I6.6)') projjjjmm
        write(ayear,'(I4.4)') projjjjmm/100
        select case ( files%init_type )
          case (4)
            pathname = str_replace(files%path_initconc, 'YYYY', ayear)
            filename = str_replace(files%file_initconc, 'YYYYMM', adate)
            filename = trim(pathname)//trim(filename)
          case (5)
            filename = str_replace(files%file_initconc, 'YYYYMM', adate)
            filename = trim(files%path_initconc)//trim(filename)
        end select
        inquire(file=trim(filename),exist=lexist)
        if(lexist) then
          varname = files%varname_init
          call get_initconc(files, config, filename, ntime, proconcini)
        else
          write(logid,*) 'WARNING: cannot find ',filename
          write(logid,*) 'WARNING: next background file missing, copy present into next to avoid 0 value'
          proconcini = concini
        endif
      endif   
    endif

    ! calculate contribution of initial concentration to receptor
    do kz = 1, nzgrid
      nz = 0
      do while(.true.)
        nz = nz + 1
        if ( outheight(kz).lt.cini_alt(nz) ) exit
      end do
      do jy = 1, nygrid
        n = 0
        do while(.true.)
          n = n + 1
          if ( gbl_lat(jy).lt.cini_lat(n) ) exit
        end do
        do ix = 1, nxgrid
          ! interpolate between concentrations
          if( (jdates(i) - trajdays).lt.jdmid ) then
            conc = preconcini(ix,jy,kz,1) + (concini(ix,jy,kz,1) - preconcini(ix,jy,kz,1)) * &
                     real((jdates(i) - trajdays) - jdpre, kind=4)/30.
          else
            conc = concini(ix,jy,kz,1) + (proconcini(ix,jy,kz,1) - concini(ix,jy,kz,1)) * &
                     real((jdates(i) - trajdays) - jdmid, kind=4)/30.
          endif
          cini(i,(nz-1)*nycini+n) = cini(i,(nz-1)*nycini+n) + gridini(ix,jy,kz)*conc
        end do
      end do
    end do

10  continue

    lastjjjjmm = jjjjmm
    prevmonth = month

  end do

  ! reorder cini for observations
  do i = 1, nobs
    obs%cini(ind(i),:) = cini(i,:)
  end do

end subroutine init_cini_month


